from xml.sax import make_parser, handler
import sys

class DataHandler:
    def __init__(self):
        self.stack      = []
        self.vars       = {}
        self.rules_graph= {}
        self.first_rule = "@@Please specify a starting rule@@"

class XMLMakefileHandler(handler.ContentHandler):
    def __init__(self, data_handler, verbose = 0):
        self.data_handler = data_handler
        self.__ID       = 0
        self.__cur_elt  = ""
        self.__verbose  = verbose
        self.__dic_s    = [
            ["abt_recipe", "makefile", "start_rule",
             self.__r_start]
            ]
        self.__dic      = [
            ["abt_recipe", "makefile", "var",
             self.__r_var],
            ["abt_recipe", "makefile", "rule", "prerequisite",
             self.__r_prerequisite],
            ["abt_recipe", "makefile", "rule", "actions",
             self.__r_actions],
            ["abt_recipe", "makefile", "rule", "doc",
             self.__r_doc]
            ]

    def startElement(self, name, attrs):
        self.data_handler.stack.append([name, attrs])

    def endElement(self, name):
        for pattern in self.__dic:
            if [i[0] for i in self.data_handler.stack] == pattern[:-1]:
                pattern[-1]()
        self.data_handler.stack.pop()

    def characters(self, data):
        self.data_handler.stack[-1].extend(data.split("\n"))
        for pattern in self.__dic_s:
            if [i[0] for i in self.data_handler.stack] == pattern[:-1]:
                pattern[-1]()

    def __r_start(self):
        self.data_handler.first_rule = self.data_handler.stack[-1][2]

    def __r_var(self):
        i = 2
        cur_line = self.data_handler.stack[-1][i]
        final_line = ""
        for cur_line in self.data_handler.stack[-1][2:]:
            if cur_line == "":
                continue
            if cur_line[-1] != "\\":
                break
            final_line += cur_line[:-1]
        self.data_handler.vars[self.data_handler.stack[-1][1]["name"]] = \
             final_line

    def __r_prerequisite(self):
        rule_name = self.data_handler.stack[-2][1]["name"]
        if not self.data_handler.rules_graph.has_key(rule_name):
            self.data_handler.rules_graph[rule_name] = [[],[], 0]
        self.data_handler.rules_graph[rule_name][0].append(
            self.data_handler.stack[-1][2])

    def __r_doc(self):
        if self.__verbose:
            rule_name = self.data_handler.stack[-2][1]["name"]
            print "%-15s - %s"% (rule_name, self.data_handler.stack[-1][2])

    def __r_actions(self):
        rule_name = self.data_handler.stack[-2][1]["name"]
        actions = []
        if not self.data_handler.rules_graph.has_key(rule_name):
            self.data_handler.rules_graph[rule_name] = [[],[], 0]
        for i in self.data_handler.stack[-1][2:]:
            if i.strip():
                if self.data_handler.stack[-1][1]["os"] == "unix":
                    a = "&" + i.strip()
                elif self.data_handler.stack[-1][1]["os"] == "win":
                    a = "%" + i.strip()
                else:
                    a = i.strip()
                if self.data_handler.stack[-1][1].has_key("mute") \
                   and self.data_handler.stack[-1][1]["mute"] == "yes":
                    a = "@" + a
                actions.append(a + "\n")
        self.data_handler.rules_graph[rule_name][1].extend(actions)

class XMLMakefileParser:
    def __init__(self, file, verbose = 0):
        self.__file     = file
        self.__verbose  = verbose

    def parse(self):
        self.data_handler = DataHandler()
        self.handler = make_parser()
        self.handler.setContentHandler(XMLMakefileHandler(self.data_handler,
                                                          self.__verbose))
        self.handler.setFeature("http://xml.org/sax/features/namespaces", 0)
        self.handler.parse(self.__file)
        self.rules_graph        = self.data_handler.rules_graph
        self.vars               = self.data_handler.vars
        self.first_rule         = self.data_handler.first_rule

if __name__ == "__main__":
    parser = XMLMakefileParser(sys.argv[1])
    parser.parse()
    print parser.vars
    print parser.data_handler.rules_graph

