#!/usr/bin/env python

from xml.sax            import make_parser, handler
import sys
from compil             import Project
from minimake           import MiniMake
from exit_status        import GL_STATUS

class DataHandler:
    def __init__(self):
        self.name       = ""
        self.version    = ""
        self.__ID       = 0
        self.__cur_elt  = ""
        self.stack      = []
        self.dep_bin    = []
        self.dep_lib    = []
        self.dep_ask    = []

class RecipeHandler(handler.ContentHandler):
    def __init__(self, data_handler):
        self.data_handler = data_handler
        self.data_handler.__ID       = 0
        self.data_handler.__cur_elt  = ""
        self.__dic_s    = [
            ["abt_recipe", "project", "name", self.__r_name],
            ["abt_recipe", "project", "version", self.__r_version]
            ]
        self.__dic_a    = [
            ["abt_recipe", "project", "dependence_bin", self.__r_dep_bin],
            ["abt_recipe", "project", "dependence_lib", self.__r_dep_lib],
            ["abt_recipe", "project", "dependence_ask", self.__r_dep_ask]
            ]
        self.data_handler.stack      = []
        self.data_handler.dep_bin    = []
        self.data_handler.dep_lib    = []
        self.data_handler.dep_ask    = []


    def startElement(self, name, attrs):
        self.data_handler.stack.append([name, attrs])
        for pattern in self.__dic_a:
            if [i[0] for i in self.data_handler.stack] == pattern[:-1]:
                pattern[-1]()
        self.data_handler._elems = 0
        self.data_handler._attrs = 0
        self.data_handler._elem_types = {}
        self.data_handler._attr_types = {}

    def endElement(self, name):
        self.data_handler.stack.pop()

    def characters(self, data):
        if self.data_handler.stack == []:
            return
        self.data_handler.stack[-1].append(data)
        for pattern in self.__dic_s:
            if [i[0] for i in self.data_handler.stack] == pattern[:-1]:
                pattern[-1]()

    def __r_name(self):
        self.data_handler.name = self.data_handler.stack[-1][2]

    def __r_version(self):
        self.data_handler.version = self.data_handler.stack[-1][2]

    def __r_dep_bin(self):
        version = "0"
        if self.data_handler.stack[-1][1].has_key("version"):
            version = self.data_handler.stack[-1][1]["version"]
        self.data_handler.dep_bin.append(
            (self.data_handler.stack[-1][1]["name"], version))

    def __r_dep_lib(self):
        version = "0"
        option  = ""
        if self.data_handler.stack[-1][1].has_key("version"):
            version = self.data_handler.stack[-1][1]["version"]
        if self.data_handler.stack[-1][1].has_key("version"):
            version = self.data_handler.stack[-1][1]["version"]
        if self.data_handler.stack[-1][1].has_key("option") \
               and self.data_handler.stack[-1][1]["option"] == "yes":
            option = "option"
        self.data_handler.dep_lib.append(
            (self.data_handler.stack[-1][1]["name"], version,
             self.data_handler.stack[-1][1]["compiler"], option))

    def __r_dep_ask(self):
        self.data_handler.dep_ask.append(
            (self.data_handler.stack[-1][1]["name"],
             self.data_handler.stack[-1][1]["question"]))

class AbtRecipe:
    def __init__(self, file):
        self.__file     = file

    def parse(self):
        self.data_handler = DataHandler()
        self.handler = make_parser()
        self.handler.setContentHandler(RecipeHandler(self.data_handler))
        self.handler.setFeature("http://xml.org/sax/features/namespaces", 0)
        self.handler.parse(self.__file)

    def run_abt(self):
        proj = Project(self.data_handler.name, self.data_handler.version,
                       self.data_handler.dep_bin, self.data_handler.dep_lib,
                       self.data_handler.dep_ask)
        self.verbose, self.filename = proj.parse_cmd_line()
        if self.verbose:
            vars = {}
        else:
            vars = proj.check()
        mini_make = MiniMake(sys.argv, vars, 0, proj.args)
        mini_make.parse(self.verbose, self.filename)
        if not self.verbose:
            mini_make.run()
        sys.exit(GL_STATUS)

if __name__ == "__main__":
    recipe = AbtRecipe("abt-recipe.xml")
    recipe.parse()
    recipe.run_abt()
