import os
import sys
from getopt     import getopt, GetoptError
from detect     import *
import cache

class Project:
    def __init__(self, name, version, prog_list = [], misc_list = [],
                 ask_user = []):
        self.name = name
        self.version = version
        self.prog_list = prog_list
        self.misc_list = misc_list
        self.ask_user = ask_user
        self.args = []
        self.vars = {}
        self.vars["PROJECT_NAME"] = name
        self.vars["PROJECT_VERSION"] = version
        self.vars["BUILDER_MINIMAKE"] = os.path.join(os.getcwd(), \
             os.path.dirname(__file__), "minimake.py")
        self.vars["BUILDER_RM"] = os.path.join(os.getcwd(), \
             os.path.dirname(__file__), "builder_rm.py")
        self.vars["BUILDER_CP"] = os.path.join(os.getcwd(), \
             os.path.dirname(__file__), "builder_cp.py")
        self.vars["BUILDER_FILE"] = os.path.join(os.getcwd(), __file__)
        self.vars["BUILDER_CWD"] = os.getcwd()
        if os.name == 'java':
            self.vars["BUILDER_INTERPRETER"] = "jython"
        else:
            self.vars["BUILDER_INTERPRETER"] = sys.executable
        self.vars["INSTALLDIR"] = "/usr/local"

    def _print_help(self):
        print """Artenum Build Tool - www.artenum.com
Maxime Biais - <maxime.biais@artenum.com>
use: abt [options] [target]

options:
   -h                   - Print this help and exit
   -l                   - Print all documented rules and exit
   -p path1:path2       - Specify paths to search in for binaries
   -i path              - Set the INSTALLDIR variable to path
   -f filename          - Use filename recipe instead of the default
                          abt-recipe.xml
   -c                   - don't use cached data - recheck for dependencies
                          and rewrite cache file (abt.cache)

target: Specify a rule to build (this rule must be defined in the
recipe). if not precized the <start_rule> from the recipe will be
executed.
"""

    def parse_cmd_line(self):
        try:
            opt, args = getopt(sys.argv[1:], "hlcp:i:f:")
        except GetoptError, e:
            print >> sys.stderr, "Error:", e
            self._print_help()
            sys.exit(1)
        filename = ""
        for i in opt:
            if i[0] == "-p":
                os.environ["PATH"] = os.environ["PATH"] + os.pathsep + i[1]
            if i[0] == "-i":
                self.vars["INSTALLDIR"] = i[1]
            if i[0] == "-f":
                filename = i[1]
            if i[0] == "-l":
                return 1, filename
            if i[0] == "-h":
                self._print_help()
                sys.exit(0)
            if i[0] == "-c":
                cache.RECHECK = 1
        self.args = args
        return 0, filename

    def setprog(self, name, version):
        self.prog_list.append((name, version))

    def miscprog(self, name, version, executable):
        self.prog_list.append((name, version, executable))

    def check(self):
        checked_vars = {}
        if cache.is_cache_file() and cache.RECHECK == 0:
            print "Using cache file, no need to check for dependencies"
            self.vars.update(cache.load())
            return self.vars
        for i in self.prog_list:
            detect = DetectProg(*i)
            tmpres = detect.check()
            if tmpres:
                checked_vars[i[0]] = tmpres
        self.vars.update(checked_vars)
        for i in self.misc_list:
            if len(i) == 4:
                opt = i[3]
            else:
                opt = ""
            if (self.vars.has_key(i[2])):
                detect = DetectMisc(i[0], i[1], self.vars[i[2]], opt)
            else:
                detect = DetectMisc(i[0], i[1], None, opt)
            tmpres = detect.check()
            if tmpres:
                checked_vars[i[0]] = "1"
        for i in self.ask_user:
            print i[1], "(%s)" % i[0] + ": "
            tmpres = raw_input()
            checked_vars[i[0]] = tmpres
        cache.save(checked_vars)
        self.vars.update(checked_vars)
        return self.vars

