import os
import sys
import checkversion
import checkmisc
import compilutils
import exit_status

# checkversion.python can be used to check the jython version
EXEC_LIB = {
    "jython"    : (["jython", "jython2.0", "jython.bat"], checkversion.python),
    "jythonc"   : (["jythonc", "jythonc.bat"], checkversion.dummy_true),
    "python"    : (["python", "python2", "python2.3", "python2.2", "python2.1",
                    "python2.0", "python1.6", "python1.5", "python.exe"],
                   checkversion.python),
    "java"      : (["java", "kaffe", "java.exe"], checkversion.java),
    "javac"     : (["javac", "gcj", "java.exe"], checkversion.dummy_true),
    "gmsh"      : (["gmsh", "gmsh.exe"], checkversion.dummy_true),
    "medit"     : (["medit", "medit-2.2-linux", "medit.exe",
                    "medit-2.2-win.exe"], checkversion.dummy_true),
    "tetgen"    : (["tetgen", "tetgen.exe"], checkversion.dummy_true)
    }

MISC_LIB = {
    "jython-vtk"     : (checkmisc.jyvtk, checkmisc.dummy_true),
    "jython-picup3d" : (checkmisc.jypicup3d, checkmisc.dummy_true)
    }

class DetectProg:
    def __init__(self, name, min_version):
        self.name = name
        self.version = min_version
        self.min_version = compilutils.hexify_version(min_version)
        self.exec_list = []
        self.the_one = None

    def __test_exec(self, execname):
        binpath = os.environ["PATH"].split(os.pathsep)
        for i in binpath:
            test_path = os.path.join(i, execname)
            if os.path.isfile(test_path):
                self.exec_list.append(test_path)

    def __test_version(self, execname):
        return EXEC_LIB[self.name][1](execname, self.min_version)

    def __build_exec_list(self):
        for i in EXEC_LIB[self.name][0]:
            self.__test_exec(i)

    def check(self):
        if self.version == "0":
            version_print = ""
        else:
            version_print = " >= " + self.version
        print "checking for", self.name + version_print , "-->",
        sys.stdout.flush()
        self.__build_exec_list()
        for i in self.exec_list:
            if self.__test_version(i):
                self.the_one = i
                print self.the_one
                return self.the_one
        print "not found"
        sys.exit(exit_status.GL_NOTFOUND)
        return self.the_one

class DetectMisc:
    def __init__(self, name, min_version, executable, opt):
        self.name = name
        self.executable = executable
        self.version = min_version
        self.min_version = compilutils.hexify_version(min_version)
        self.opt = opt

    def check(self):
        if self.version == "0":
            version_print = ""
        else:
            version_print = " >= " + self.version
        print "checking for", self.name + version_print, "with", \
              self.executable,
        res = 0
        if self.executable:
            res = MISC_LIB[self.name][0](self.executable) \
                  and MISC_LIB[self.name][1]()
        if res:
            print "--> OK"
        else:
            print "--> FAILED"
            if self.opt != "option":
                sys.exit(exit_status.GL_NOTFOUND)
            else:
                print "WARNING %s was not found" % self.name
                print "%s is optional: press a key to continue" \
                      % self.name
                raw_input()
        return res

