#!/usr/bin/env python

import sys, os
from makefile_lexer     import MakefileLexer
from makefile_parser    import MakefileParser

class Converter:
    def __init__(self, filename):
        self.filename = filename

    def parse(self):
        scanner = MakefileLexer(self.filename)
        self.parser = MakefileParser(scanner.scan())
        self.parser.parse()

    def __str_prologue(self):
        return """<!-- AUTOCONVERTED ABT-RECIPE FROM MAKEFILE -->
<abt_recipe>
  <project>
    <name>FIXME</name>
    <version>FIXME</version>
    <!-- <dependence_bin name="FIXME" version="FIXME" /> -->
    <!-- <dependence_lib name="FIXME" compiler="FIXME" option="FIXME"/> -->
    <!-- <dependence_ask name="FIXME" question="FIXME"/> -->
  </project>
  <makefile>
"""

    def __str_epilogue(self):
        return """  </makefile>
</abt_recipe>"""

    def __str_vars(self):
        """
        return the serialized variables
        """
        str = ""
        for i in self.parser.vars.keys():
            str += "    <var name=\"" + i + "\">" \
                   + self.parser.vars[i].strip() \
                   + "</var>" + os.linesep
        return str

    def __str_rules(self):
        """
        return the serialized rules
        """
        str = ""
        for i in self.parser.rules_graph.keys():
            str += "    <rule name=\"" + i + "\">" + os.linesep
            if self.parser.rules_graph[i][0]:
                for pre in self.parser.rules_graph[i][0]:
                    str += "      <prerequisite>" + pre + "</prerequisite>" \
                           + os.linesep
            if self.parser.rules_graph[i][1]:
                str += "      <actions os=\"all\">" + os.linesep
                for action in self.parser.rules_graph[i][1]:
                    str += "        " + action + os.linesep
                str += "      </actions>" + os.linesep
            str += "    </rule>" + os.linesep + os.linesep
        return str

    def __str_start_rule(self):
        """
        return the serialized start_rule
        """
        return os.linesep + "    <start_rule>" + self.parser.first_rule \
               + "</start_rule>" + os.linesep + os.linesep

    def __str__(self):
        str = self.__str_prologue()
        str += self.__str_vars()
        str += self.__str_start_rule()
        str += self.__str_rules()
        str += self.__str_epilogue()
        return str

    # jython hack
    def tostdout(self):
        print self.__str__()

    def tofile(self, fileout):
        stream = open(fileout, "w")
        stream.write(self.__str__())
        stream.close()

if __name__ == "__main__":
    if len(sys.argv) == 2:
        converter = Converter(sys.argv[1])
        converter.parse()
        converter.tostdout()
    elif len(sys.argv) == 3:
        converter = Converter(sys.argv[1])
        converter.parse()
        converter.tofile(sys.argv[2])
    else:
        print >> sys.stderr, "Usage: makefile2abt-recipe Makefile [fileOUT]"
