import re
import sys
import exit_status

class MakefileLexer:
    def __init__(self, file):
        # Warning! The order of tokens is important (ERROR must be the last)
        self.tokens = [
          ("TOK_NAME",         "([a-zA-Z_\.\$][a-zA-Z0-9_\.\$\(\)]*)", self.__dummy),
          ("TOK_COLON",        "(:)",                        self.__dummy),
          ("TOK_ASSIGNED",     "=(.*)",                      self.__dummy),
          ("TOK_ACTION",       "\t([^:\n].*)",               self.__dummy),
          ("TOK_EOL",          "(\n\r|\r\n|\n)",             self.__inc_line),
          ("SEPARATORS",       "(\s+)",                      self.__dummy),
          ("ERROR",            "(.*)",                       self.__scan_error)
        ]
        self.replacements = {
            "##.*[\n|\n\r|\r\n]":        ""
            }
        self.file = file
        self.line_counter = 1
        self.res = []

    # Lexer handlers
    def __dummy(self, re_name, re_code, parsed):
        """Do nothing"""
        pass

    def __inc_line(self, re_name, re_code, parsed):
        """Increment the line counter"""
        self.line_counter += 1

    def __scan_error(self, re_name, re_code, parsed):
        """Print the error, with location"""
        print >>sys.stderr, "Scan error: line " + str(self.line_counter) \
              + " \"" + parsed + "\"" + ". The parser will ignore this line."
        exit_status.GL_STATUS = exit_status.GL_SCAN

    def scan(self):
        """Scan self.file following tokens rules defined in self.tokens"""
        self.__pre_scan()
        i = 0
        while len(self.stream) != 0:
            for myre in self.tokens:
                tmpre = re.compile(myre[1]).match(self.stream)
                if tmpre:
                    size = len(tmpre.group(0))
                    self.stream = self.stream[size:]
                    i += size
                    # Call the associated function
                    myre[2](myre[0], myre[1], tmpre.group(1))
                    if myre[0] != "SEPARATORS" and myre[0] != "ERROR":
                        self.res.append((myre[0], tmpre.group(1)))
                    break
        return self.res

    def __pre_scan(self):
        self.__readfile()
        for i in self.replacements.keys():
            self.stream = re.sub(i, self.replacements[i], self.stream)


    def __readfile(self):
        """Open self.file and read it into the string self.stream"""
        try:
            file = open(self.file)
            self.stream = file.read()
        except IOError:
            exit_status.GL_STATUS = exit_status.GL_SCAN
            raise

if __name__ == "__main__":
    """Testing purpose"""
    lex = MakefileLexer("Makefile")
    print lex.scan()
    sys.exit(exit_status.GL_STATUS)
