import re
import sys

import exit_status

# FIXME: derecursive the parser

# minimake grammar:
#
# makefile: /* empty */
#         | makefile assignement
#         | makefile rule
#         | makefile EOL
#
# assignment: NAME ASSIGNED
#
# rule: NAME COLON prerequisites EOL actions
#
# actions: /* empty */
#        | ACTION actions
#        | ACTION EOL
#        | ACTION EOF
#
# prerequisites: /* empty */
#              | NAME prerequisites


class MakefileParser:
    def __init__(self, tokens_list, debug = 0):
        self.tokens_list = tokens_list
        self.__get_cur_tokens()
        self.debug_trace = debug
        self.vars = {}
        self.rules_graph = {}
        self.first_rule = None

    def __parse_trace(self, str):
        """Print the parse trace (debug)"""
        if self.debug_trace:
            print >>sys.stderr, str

    def __get_cur_tokens(self):
        """
        Get the current tokens from the current list, if the list is
        to small then TOK_EOF will be added
        """
        if len(self.tokens_list) == 1:
            self.cur_tokens = (self.tokens_list[0][0], ("TOK_EOF"))
        elif len(self.tokens_list) == 0:
            self.cur_tokens = (("TOK_EOF"), ("TOK_EOF"))
        else:
            self.cur_tokens = (self.tokens_list[0][0], self.tokens_list[1][0])

    def __parse_error(self):
        """Print the error, and put the good exit status"""
        print >>sys.stderr, "Parse error: <%s> not expected" \
              % self.tokens_list[0][1]
        exit_status.GL_STATUS = exit_status.GL_PARSE

    def __eat(self, token_to_eat):
        """
        Eat the current token, modify the tokens list and update the
        current tokens
        """
        if len(self.tokens_list) == 0:
            return
        if self.tokens_list[0][0] == token_to_eat:
            self.__parse_trace("%-25s: %-15s: %s" %
                               ("Eat", token_to_eat,
                                self.tokens_list[0][1].strip()))
            self.tokens_list = self.tokens_list[1:]
            self.__get_cur_tokens()
        else:
            self.__parse_error()

    def __r_makefile(self):
        """makefile rule (see the grammar in the beginning of the file)"""
        self.__parse_trace("%-25s: %s" % ("rule makefile",
                                          str(self.cur_tokens)))
        if self.cur_tokens[0] == "TOK_NAME" \
               and self.cur_tokens[1] == "TOK_ASSIGNED":
            self.__r_assignement()
            self.__r_makefile()
        elif self.cur_tokens[0] == "TOK_NAME" \
                 and self.cur_tokens[1] == "TOK_COLON":
            self.__r_rule()
            self.__r_makefile()
        elif self.cur_tokens[0] == "TOK_EOL":
            self.__eat("TOK_EOL")
            self.__r_makefile()
        elif self.cur_tokens[0] == "TOK_EOF":
            return
        else:
            self.__parse_error()

    def __r_assignement(self):
        """assignement rule (see the grammar in the beginning of the file)"""
        self.__parse_trace("%-25s: %s" % ("rule assignement",
                                          str(self.cur_tokens)))
        # Add it to the variables dictionary
        self.vars[self.tokens_list[0][1]] = self.tokens_list[1][1]
        self.__eat("TOK_NAME")
        self.__eat("TOK_ASSIGNED")

    def __r_rule(self):
        """rule rule (see the grammar in the beginning of the file)"""
        self.__parse_trace("%-25s: %s" % ("rule rule",
                                          str(self.cur_tokens)))
        rule_name = self.tokens_list[0][1]
        if self.first_rule == None:
            self.first_rule = rule_name
        self.rules_graph[rule_name] = [[],[], 0]
        self.__eat("TOK_NAME")
        self.__eat("TOK_COLON")
        self.rules_graph[rule_name][0] = self.__r_prerequisites()
        self.__eat("TOK_EOL")
        self.rules_graph[rule_name][1] = self.__r_actions()

    def __r_actions(self):
        """actions rule (see the grammar in the beginning of the file)"""
        res = []
        while self.cur_tokens[0] == "TOK_ACTION":
            res.append(self.tokens_list[0][1])
            self.__eat("TOK_ACTION")
            self.__eat("TOK_EOL")
        return res

    def __r_prerequisites(self):
        """
        prerequisites rule (see the grammar in the beginning of the
        file)
        """
        res = []
        while self.cur_tokens[0] == "TOK_NAME":
            res.append(self.tokens_list[0][1])
            self.__eat("TOK_NAME")
        return res

    def parse(self):
        """Run the starting rule"""
        self.__r_makefile()

if __name__ == "__main__":
    """Testing purposes"""
    from makefile_lexer import MakefileLexer
    scanner = MakefileLexer("Makefile")
    parser = MakefileParser(scanner.scan())
    parser.parse()
