import re
import sys
import os
from exit_status        import GL_STATUS, GL_MISC
from makefile_lexer     import MakefileLexer
from makefile_parser    import MakefileParser
from XMLMakefile        import XMLMakefileParser

class MiniMake:
    def __init__(self, args, vars, debug = 0, first_rules = None):
        self.args = args
        self.token_list = []
        self.vars = vars
        self.rules_graph = {}
        self.first_rules = first_rules
        self.debug = 0

    def __add_vars(self, new_vars):
        self.vars.update(new_vars)

    def __var_expand(self, str):
        # Always test if str if not NULL to avoid a compatibilty
        # problem with concatening NULL string with jython 2.1
        oldstr = None
        while str and re.search("\$\(.*\)", str) and oldstr != str:
            oldstr = str
            for i in self.vars.keys():
                myre = re.compile("(.*)(\$\(%s\))(.*)" % i)
                re_res = myre.match(str)
                if re_res:
                    str = ""
                    if re_res.group(1):
                        str += re_res.group(1)
                    if self.vars[i]:
                        str += self.vars[i]
                    if re_res.group(3):
                        str += re_res.group(3)
        # Remove not known variables
        if str:
            str = re.sub("\$\(.*\)", "", str)
        return str

    def __var_substitution(self):
        for i in self.rules_graph.keys():
            if self.__var_expand(i) != i:
                self.rules_graph[self.__var_expand(i)] = self.rules_graph[i]
            for j in range(0, len(self.rules_graph[i][1])):
                self.rules_graph[i][1][j] = \
                    self.__var_expand(self.rules_graph[i][1][j])
            for j in range(0, len(self.rules_graph[i][0])):
                self.rules_graph[i][0][j] = \
                    self.__var_expand(self.rules_graph[i][0][j])
        for i in self.vars.keys():
            self.vars[i] = self.__var_expand(self.vars[i])

    def __need_update(self, main_rule, prerequisites):
        """
        Check the file date and compare to the prerequisites ones, if
        the main rule is newer than all the prerequisites then return
        False else True
        """
        if not os.path.isfile(main_rule):
            return 1
        main_date = os.stat(main_rule)[8]
        if not prerequisites:
            return 1
        for pre in prerequisites:
            if not os.path.isfile(pre):
                return 1
            if os.stat(pre)[8] > main_date:
                return 1
        print "No need to build:", main_rule, \
              "target is newer than prerequisites"
        return 0

    def __run_rule(self, rule_name):
        if os.path.isfile(rule_name) \
               and not self.rules_graph.has_key(rule_name):
            return
        if not self.rules_graph.has_key(rule_name):
            print >> sys.stderr, "Rule <%s> does not exist" \
                  % rule_name
            sys.exit(GL_MISC)
        if self.rules_graph[rule_name][2] == 1:
            print >> sys.stderr, "circular dependency with rule <%s>" \
                  % rule_name
            sys.exit(GL_MISC)
        self.rules_graph[rule_name][2] = 1
        if self.__need_update(rule_name, self.rules_graph[rule_name][0]) == 0:
            return
        for prerequisite in self.rules_graph[rule_name][0]:
            self.__run_rule(prerequisite)
        for i in self.rules_graph[rule_name][1]:
            toprint = 1
            if i[0] == "@":
                i = i[1:]
                toprint = 0
            if i[0] == "%":
                i = i[1:]
                if sys.platform.startswith('win'):
                    if toprint:
                        print i
                    os.system(i)
            elif i[0] == "&":
                i = i[1:]
                if not sys.platform.startswith('win'):
                    if toprint:
                        print i
                    os.system(i)
            else:
                if toprint:
                    print i
                os.system(i)


    def parse(self, verbose = "", filename = ""):
        if not os.path.isfile("abt-recipe.xml"):
            default_files = ["makefile", "Makefile"]
            for i in default_files:
                try:
                    lexer = MakefileLexer(i)
                    self.token_list = lexer.scan()
                except IOError:
                    pass
                else:
                    break
            parser = MakefileParser(self.token_list, self.debug)
        else:
            parser = XMLMakefileParser(filename or "abt-recipe.xml", verbose)
        parser.parse()
        self.__add_vars(parser.vars)
        self.rules_graph = parser.rules_graph
        if not self.first_rules:
            self.first_rules = [parser.first_rule]

    def run(self):
        self.__var_substitution()
        for i in self.first_rules:
            self.__run_rule(i)
        pass

if __name__ == "__main__":
    import sys
    if len(sys.argv) >= 2:
        mini_make = MiniMake(sys.argv, {}, 0, sys.argv[1:])
    else:
        mini_make = MiniMake(sys.argv, {}, 0)
    mini_make.parse()
    mini_make.run()
    sys.exit(GL_STATUS)

